<?php

/**
 * Author：晴玖天
 * Creation：晓风 2024/08/30 修
 * Filename：jubijia.php
 * 聚比价
 * QQ交流群960640976
 */

namespace lib\supply;

use Medoo\DB\SQL;

class jubijia
{
    /**
     * @var array
     * 对接配置信息
     */
    public static $Config = [
        'name' => '聚比价',
        'image' => '../assets/img/def.png',
        'PriceMonitoring' => 1, //是否支持价格监控
        'BatchShelves' => 1, //批量上架
        'help' => '聚比价对接插件V12000，QQ交流群960640976 -- 晓风',
        'ip' => -1,
        'field' => [ //添加货源的字段名称
            'url' => [
                'name' => '对接网站',
                'tips' => '请填写包含http(s):// 和 / 的域名地址!',
                'type' => 1,
            ],
            'username' => [
                'name' => '对接账号',
                'tips' => '请填写对接账号',
                'type' => 1,
            ],
            'password' => [
                'name' => '对接密匙',
                'tips' => '请填写对接密匙',
                'type' => 1,
            ],
        ],
        'InputField' => [ //选择对接商品时的输入框字段
            'ProductList' => [
                'type' => 2,
                'name' => '可对接商品', //下拉列表名称
                'request' => [ //提交的是当前选择参数数据类的值 + 当前的货源社区ID加类型键值！
                    'gid' => false,
                    'type' => 1, //固定提交的参数
                    'controller' => 'ProductDetails', //请求的控制器名称
                ],
                'GetListData' => [ //获取列表时的请求数据，提交的是当前所选对接社区内的值+当前已选参数的值
                    'id' => false,
                    'type' => 1,
                    'controller' => 'GoodsList', //请求的控制器名称
                ],
                'RequestDataSources' => '-1', //点击列表参数请求替换源，data() { return (全部可用) }，-1为内置源GoodsData，其他为App[xxx]
                'GetListDataSources' => 'DockingData', //列表请求替换源，data() { return (全部可用) }
                'BtnName' => '获取数据', //获取列表时的按钮名称
                'button' => [ //辅助按钮
                    [
                        'name' => '获取最新数据',
                        'GetListDataSources' => 'DockingData', //请求替换源，data() { return (全部可用) }
                        'class' => 'badge badge-primary-lighten',
                        'request' => [
                            'id' => false,
                            'type' => 2,
                            'controller' => 'GoodsList', //请求的控制器名称
                        ],
                    ]
                ]
            ],
            'gid' => [
                'type' => 1,
                'name' => '商品ID',
                'reminder' => '请点击获取数据后，选择对应商品获取'
            ],
            'parameter' => [
                'type' => 1,
                'name' => '对接参数',
                'reminder' => '请点击获取数据后，选择对应商品获取'
            ]
        ]
    ];

    /**
     * @param $Data //用户提交的数据
     * @param $Source //货源详情
     * @param $DataSo //提交按钮详情
     */
    public static function AdminOrigin($Data = [], $Source = [], $DataSo = [])
    {
        $plugin_id = 12;
        $plugin_name = '聚比价对接';
        switch ($Data['controller']) {
            case 'GoodsList': //获取可对接的商品列表
                return self::GoodsList($Data);
            case 'ProductDetails': //获取商品详情
                return self::ProductDetails($Data);
            case 'Submit': //对接下单
                return self::Submit($Data['Order'], $Data['Goods'], $Data['Supply']);
            case 'Query': //查询订单
                return self::Query($Data['Order'], $Data['Supply']);
            case 'CommodityStatus': //商品状态详情
                return self::CommodityStatus($Data['Goods'], $Data['Supply']);
            case 'CommodityAnalysis': //解析商品配置信息
                return self::CommodityAnalysis($Data);
        }
        return [
            'code' => -1,
            'msg' => '请求错误',
        ];
    }

    /**
     * @param $Data
     * @return array
     * 获取可对接商品列表
     */
    public static function GoodsList($Data)
    {
        if ($Data['type'] != 2 && !empty($_SESSION['JUBIJIAGoodsList_' . $Data['id']])) {
            $GoodsList = [];
            foreach ($_SESSION['JUBIJIAGoodsList_' . $Data['id']] as $v) {
                $GoodsList[] = [
                    'gid' => $v['id'],
                    'name' => $v['name'] . " | " . $v['price'] . '元',
                    'cid' => $v['categoryId'],
                ];
            }
            return [
                'code' => 1,
                'msg' => '可对接商品列表获取成功！',
                'data' => $GoodsList
            ];
        }

        $DB = SQL::DB();
        $SourceData = $DB->get('shequ', ['url', 'username', 'password'], [
            'id' => $Data['id'],
        ]);
        if (!$SourceData) {
            return [
                'code' => -1,
                'msg' => '对接社区数据获取失败，请检查此对接社区是否存在！',
            ];
        }
        $Post = [
            'userId' => $SourceData['username'],
        ];
        $Post['sign'] = self::GetSign($Post, $SourceData['password']);
        $SourceData['url'] = StringCargo::UrlVerify($SourceData['url'], 3);

        $CurlData = Api::Curl($SourceData['url'] . "/api/open/all-product", $Post);
        $CurlDataJson = json_decode($CurlData, true);
        if (empty($CurlDataJson)) {
            return [
                'code' => -1,
                'msg' => '对接网站：' . $SourceData['url'] . ' 打开失败，请检查域名是否有误，或手动Ping测试连通率',
            ];
        }
        if ($CurlDataJson['code'] === 200) {
            $_SESSION['JUBIJIAGoodsList_' . $Data['id']] = $CurlDataJson['data'];
            $GoodsList = [];
            foreach ($CurlDataJson['data'] as $v) {
                $GoodsList[] = [
                    'gid' => $v['id'],
                    'name' => $v['name'] . " | " . $v['price'] . '元',
                    'cid' => $v['categoryId'],
                ];
            }
            return [
                'code' => 1,
                'msg' => '可对接商品列表获取成功！',
                'data' => $GoodsList
            ];
        }

        return [
            'code' => -1,
            'msg' => $CurlDataJson['msg'],
        ];
    }

    /**
     * @param $array
     * @return string
     * 直客签名
     */
    public static function GetSign($param, $key = '')
    {
        $signPars = '';
        ksort($param);
        foreach ($param as $k => $v) {
            $k = trim($k);
            $v = trim($v);
            if ($k !== 'sign' && $v !== '') {
                $signPars .= $k . '=' . $v . '&';
            }
        }
        $signPars = trim($signPars, '&');
        $signPars .= $key;
        return md5($signPars);
    }

    /**
     * @param $Data
     * @return array
     * 获取商品详情
     */
    public static function ProductDetails($Data)
    {
        $DB = SQL::DB();
        $SourceData = $DB->get('shequ', ['url', 'username', 'password'], [
            'id' => $Data['sqid'],
        ]);
        if (!$SourceData) {
            return [
                'code' => -1,
                'msg' => '对接社区数据获取失败，请检查此对接社区是否存在！',
            ];
        }
        $Post = [
            'userId' => $SourceData['username'],
            'id' => $Data['gid'],
        ];
        $Post['sign'] = self::GetSign($Post, $SourceData['password']);
        $SourceData['url'] = StringCargo::UrlVerify($SourceData['url'], 3);

        $CurlData = Api::Curl($SourceData['url'] . "/api/open/product/detail", $Post);

        $CurlDataJson = json_decode($CurlData, true);
        if (empty($CurlDataJson)) {
            return [
                'code' => -1,
                'msg' => '对接网站：' . $SourceData['url'] . ' 打开失败，请检查域名是否有误，或手动Ping测试连通率',
            ];
        }

        if ($CurlDataJson['code'] !== 200) {
            return [
                'code' => -1,
                'msg' => $CurlDataJson['msg']
            ];
        }
        $Goods = $CurlDataJson['data'];
        $Input = [];
        $Goods['attach'] = json_decode($Goods['attach'], true);
        //链接地址|2222{dd,aa}
        foreach ($Goods['attach'] as $key => $val) {
            if ($val['type'] == 2) {
                //下拉
                $Input[0][] = $val['name'] . "{" . $val['data'] . "}";
            } else {
                $Input[0][] = $val['name'];
            }
            $Input[1][] = 'input' . ($key + 1);
        }
        if (empty($Goods['attach'])) {
            $Input[0][] = '';
            $Input[1][] = '无';
        }
        // die(json_encode( $Goods,JSON_UNESCAPED_UNICODE));
        return [
            'code' => 1,
            'msg' => '对接参数自动填写成功！',
            'data' => [
                'name' => $Goods['name'],
                'image' => $Goods['image'],
                'alert' => '',
                'docs' => $Goods['details'],
                'money' => (float)$Goods['price'],
                'min' => 1,
                'max' => 99999,
                'quota' => is_numeric($Goods['quota']) ? $Goods['quota'] : 99999,
                'quantity' => 1,
                'extend' => [
                    'gid' => $Goods['id'],
                    'parameter' => implode(',', $Input[1]),
                ],
                'units' => '',
                'input' => implode('|', $Input[0]),
            ]
        ];
    }

    /**
     * @param $OrderData 订单信息
     * @param $Goods 商品信息
     * @param $TypeSupply 对接货源信息
     * 提交下单信息！
     */
    public static function Submit($OrderData, $Goods, $TypeSupply)
    {
        global $conf;
        $InputArray = json_decode($OrderData['input'], TRUE);
        if ($Goods['specification'] == 2 && $Goods['specification_type'] == 2) {
            $InputArray = RuleSubmitParameters(json_decode($Goods['specification_spu'], TRUE), $InputArray);
        }

        $GoodsData = json_decode($Goods['extend'], true); //对接数据
        $PostInput = [];
        if ($GoodsData['parameter'] != '无') {
            $Field = explode(',', $GoodsData['parameter']);
            $i = 0;
            foreach ($Field as $value) {
                $PostInput[] = $InputArray[$i];
                $i++;
            }
        }
        $header = [
            'Content-Type' => 'application/json; charset=utf-8',
        ];
        $Post = [
            'userId' => $TypeSupply['username'],
        ];
        $Post['sign'] = self::GetSign($Post, $TypeSupply['password']);
        $Body = [
            'attach' => $PostInput,
            'num' => (int)$OrderData['num'],
            'productId' => $GoodsData['gid'],
            'price' => $OrderData['price'],
            'url' => userip(),
        ];
        $SourceData['url'] = StringCargo::UrlVerify($TypeSupply['url'], 3);

        $DataCurl = Api::SuppluCurl($SourceData['url'] . "/api/open/purchase?userId=" . $TypeSupply['username'] . '&sign=' . $Post['sign'], json_encode($Body), $header);
        if (empty($DataCurl)) {
            return [
                'code' => $conf['SubmitState'],
                'docking' => 2,
                'msg' => '对接返回信息有误，请根据对接日志调试！',
                'money' => 0,
                'order' => 0,
            ];
        }

        $DataCurl = json_decode($DataCurl, TRUE);

        if (!isset($DataCurl['code'])) {
            $Msg = '下单信息获取失败!,请检查对接配置！';
            $code = $conf['SubmitState'];
            $money = 0;
            $order = -1;
            $docking = 2;
        } else {
            $Msg = $DataCurl['msg'];
            $code = ($DataCurl['code'] === 200 ? $conf['SubmitStateSuccess'] : $conf['SubmitState']);
            $money = -1;
            $order = ($DataCurl['code'] === 200 ? $DataCurl['data']['orderNumber'] : -1);
            $docking = ($DataCurl['code'] === 200 ? 1 : 2);

            if ($DataCurl['code'] == 200) {
                $money = preg_replace('/[^0-9.]/', '', $DataCurl['msg']);
                if ($money == 0) {
                    $money = -1;
                }
            }
        }
        if ($DataCurl['code'] === 200 && !empty($DataCurl['data']['kmData'])) {
            global $date;
            $DB = SQL::DB();
            $SQL[] = [
                'uid' => $OrderData['uid'],
                'gid' => $Goods['gid'],
                'code' => $InputArray[0],
                'token' => $DataCurl['data']['kmData'],
                'ip' => $OrderData['ip'],
                'order' => $OrderData['order'],
                'endtime' => $date,
                'addtime' => $date,
            ];
            $DB->insert('token', $SQL);
        }
        return [
            'code' => $code,
            'docking' => $docking,
            'msg' => $Msg,
            'money' => $money,
            'order' => $order,
        ];
    }

    /**
     * @param $Order 订单信息
     * @param $TypeSupply 对接货源信息
     */
    public static function Query($Order, $TypeSupply)
    {

        $Post = [
            'userId' => $TypeSupply['username'],
            'order' => $Order['order_id'],
        ];
        $Post['sign'] = self::GetSign($Post, $TypeSupply['password']);
        $TypeSupply['url'] = StringCargo::UrlVerify($TypeSupply['url'], 3);

        $DataCurl = Api::Curl($TypeSupply['url'] . "/api/open/order", $Post);
        // die($DataCurl);
        $DataCurl = json_decode($DataCurl, TRUE);
        if (empty($DataCurl)) {
            return false;
        }
        if ($DataCurl['code'] != 200) {
            return false;
        }
        // die(json_encode($DataCurl['data'],JSON_UNESCAPED_UNICODE));
        $Data = $DataCurl['data'][0];
        switch ($Data['status']) {
            case -1:
                $order_state = '下单失败';
                $StateNum = 3;
                break;
            case 0:
                $order_state = '处理中';
                $StateNum = 4;
                break;
            case 1:
                $order_state = '下单成功';
                $StateNum = 1;
                break;
            case 2:
                $order_state = '已退单';
                $StateNum = 5;
                break;
                // case 3:
                //     $order_state = '已后台介入';
                //     $StateNum = 6;
                //     break;
                // case 4:
                //     $order_state = '已退单';
                //     $StateNum = 5;
                //     break;
            default:
                $order_state = '未知状态';
                $StateNum = 3;
                break;
        }
        $resOrder =   $DataCurl['data'][0];
        // die(json_encode($resOrder,JSON_UNESCAPED_UNICODE));
        if (isset($resOrder['kmData']) && count(explode(',', $resOrder['kmData'])) >= 1) {
            $kamiS = $resOrder['kmData'];
            $TokenList =  explode(',', $kamiS);
            //有卡
            $DB = SQL::DB();
            $TokenGet = $DB->count('token', [
                'order' => $Order['order'],
            ]) - 0;
            if ($TokenGet < count($TokenList)) {
                global $date;
                //补卡
                $SQL = [];
                foreach ($TokenList as $v) {
                    if (empty($v)) {
                        continue;
                    }
                    $SQL[] = [
                        'uid' => $Order['uid'],
                        'gid' => $Order['gid'],
                        'code' => json_decode($Order['input'], true)[0],
                        'token' => $v,
                        'ip' => $Order['ip'],
                        'order' => $Order['order'],
                        'endtime' => $date,
                        'addtime' => $date,
                    ];
                }
                if (count($SQL) >= 1) {
                    $DB->insert('token', $SQL);
                }
            }
        }
        return [
            'ApiType' => 1,
            'ApiNum' => $Data['quantity'],
            'ApiTime' => $Data['orderTime'],
            'ApiInitial' => '无',
            'ApiPresent' => '无',
            'ApiState' => $order_state,
            'ApiError' => $Data['orderInfo'] ?? '',
            'ApiStateNum' => $StateNum,
        ];
    }

    /**
     * @param $Goods
     * @param $Supply
     * 商品详细状态
     * @return array
     */
    public static function CommodityStatus($Goods, $Supply)
    {
        $GoodsData = json_decode($Goods['extend'], true);
        if (empty($GoodsData['gid'])) {
            return [
                'code' => -1,
                'msg' => '此商品对接参数缺失，无法完成商品状态监控！',
            ];
        }
        $Supply['url'] = StringCargo::UrlVerify($Supply['url'], 3);
        $Post = [
            'userId' => $Supply['username'],
            'id' => $GoodsData['gid'],
        ];
        $Post['sign'] = self::GetSign($Post, $Supply['password']);
        $SourceData['url'] = StringCargo::UrlVerify($Supply['url'], 3);
        $CurlData = Api::Curl($SourceData['url'] . "/api/open/product/detail", $Post);

        $CurlDataJson = json_decode($CurlData, true);
        if (empty($CurlDataJson)) {
            return [
                'code' => -1,
                'msg' => '对接网站：' . $Supply['url'] . ' 打开失败，请检查域名是否有误，或手动Ping测试连通率',
            ];
        }
        if ((int)$CurlDataJson['code'] !== 200) {
            return [
                'code' => 1,
                'msg' => '数据获取成功',
                'data' => [
                    'state' => 2,
                    'money' => 0,
                    'inventory' => 0,
                ]
            ];
        }
        $GoodsDataApi = $CurlDataJson['data'];
        // die("库存：".(is_numeric($GoodsDataApi['quota'])?$GoodsDataApi['quota']:9999));
        /**
         * 解析数据
         */
        return [
            'code' => 1,
            'msg' => '数据获取成功',
            'data' => [
                'inventory' => is_numeric($GoodsDataApi['quota']) ? $GoodsDataApi['quota'] : 99999,
                'state' => 1,
                'money' => (float)$GoodsDataApi['price'],
                'specification' => false,
            ]
        ];
    }

    /**
     * @param $Data
     * @return array
     * 获取新增商品SQL数据
     */
    public static function CommodityAnalysis($Data)
    {
        global $date;
        $Goods = json_decode($Data['data'], true);
        return [
            'cid' => $Data['cid'],
            'sort' => $Data['sort'],
            'name' => $Data['name'],
            'image[JSON]' => [ImageUrl($Goods['image'])],
            'min' => $Goods['min'],
            'max' => $Goods['max'],
            'quota' => $Goods['quota'],
            'input' => $Goods['input'],
            'quantity' => $Goods['quantity'],
            'alert' => $Goods['alert'],
            'docs' => $Goods['docs'],
            'money' => $Goods['money'],
            'units' => (empty($Goods['units']) ? '个' : $Goods['units']),
            'deliver' => -1,
            'sqid' => $Data['sqid'],
            'specification' => 1,
            'specification_type' => 2,
            'specification_spu[JSON]' => [],
            'specification_sku[JSON]' => [],
            'extend[JSON]' => $Goods['extend'],
            'date' => $date,
        ];
    }
}
